#include <Adafruit_GFX.h>    // Core graphics library
#include <SPI.h>

//leds
#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
#include <avr/power.h> // Required for 16 MHz Adafruit Trinket
#endif
#define PIN      8
#define NUMPIXELS 5


#include <Adafruit_Arcada.h>
#include <Adafruit_Arcada_Def.h>
#include <Adafruit_Arcada_Filesystem.h>
#include <arcadatype.h>

#include "Map1.h"
#include "Stone.h"
#include "Player.h" 
#include "Enemy.h"
#include "MCmovement.h"
#include "EnemyAnim.h"


//function header
int** createEpos(int y);


//declaring & init global variables
Adafruit_NeoPixel pixels(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);
Adafruit_Arcada arcada;

int room; //levels 
bool releaseA = false; //for A button

int* pos = (int*)calloc(2, sizeof(int)); //player pos
Player* p;  //player
int** e_pos = createEpos(5);
//create enemies, max 5
Enemy* e1 ;
Enemy* e2 ;
Enemy* e3 ;
Enemy* e4 ;
Enemy* b1 ; //boss

File musicFile;
bool musicIsPlaying = false;
bool inMenu = true;


void setup() {
  // put your setup code here, to run once:
  pixels.begin();
  arcada.arcadaBegin();
  
  // initialize the file system on the flash (first flashing should be done via CircuitPython)
  arcada.filesysBegin(); // arcada.filesysBeginMSD() for "USB-Stick" mode

  // initialize serial connection
  Serial.begin(9600);
  Serial.println("Booting into game...");

  // configure board display and enable frame buffer mode
  arcada.displayBegin();
  arcada.setBacklight(255);
  arcada.createFrameBuffer(ARCADA_TFT_WIDTH, ARCADA_TFT_HEIGHT);
  
  // we start our loop in the menu
  inMenu = true;

  //creating objects once, init values don't matter here, values at handleMenu()
  p = new Player(pos,16,16,10); 
  e1 = new Enemy(e_pos[0],16,16,50,0,0,0,0,false); //hp=50
  e2 = new Enemy(e_pos[1],16,16,50,0,0,0,0,false);
  e3 = new Enemy(e_pos[2],16,16,50,0,0,0,0,false);

 // b1 = new Enemy(e_pos[4],16,16,200,0,0,0,0,false); //hp=200
}


void loop() {

  //main loop
  if (inMenu) {
    handleMenu();
  } else {
    inGame();
  }
  
  // render the frame buffer to the screen
  arcada.blitFrameBuffer(0, 0, true, false);

  // perform standard board functions (cooperative multitasking)
  yield();
}


/**
 * Handle menu input and rendering
 */
void handleMenu() {
  uint8_t buttonState = arcada.readButtons();
  // start pressed: load music and switch to game state
  if (buttonState & ARCADA_BUTTONMASK_START) {
    //init
    pos[0]=4; 
    pos[1]=4;
    e_pos[0][0]=100; 
    e_pos[0][1]=100;
    e_pos[1][0]=74; 
    e_pos[1][1]=93;
    e_pos[2][0]=94; 
    e_pos[2][1]=32;
  
    p -> setPlayer(pos,16,16,10); // 10hp max
    e1 -> setEnemy(e_pos[0],16,16,50,0,0,0,0,false); //hp=50
    e2 -> setEnemy(e_pos[1],16,16,50,0,0,0,0,false);
    e3 -> setEnemy(e_pos[2],16,16,50,0,0,0,0,false);
    //b1 -> setEnemy(e_pos[4],16,16,200,0,0,0,0,false); //hp=200
    room = 0;
    
    loadMusic();
    randomSeed(millis());
    inMenu = false;
  }

  // use standard rendering functions without directly updating the screen
  GFXcanvas16 *canvas = arcada.getCanvas();
  canvas->fillScreen(ARCADA_BLACK);
  
  canvas->setTextSize(2);
  
  canvas->setTextColor(ARCADA_BLUE);
  canvas->setCursor(18, 30);
  canvas->print("Losto");
  canvas->setTextColor(ARCADA_RED);
  canvas->setCursor(36, 60);
  canvas->print("Doggo");

  canvas->drawRGBBitmap(100,60 , Startscreen, 48, 64);
}


/*
 * Handle game rendering and input
 */
void inGame(){
  uint8_t bState = arcada.readButtons();
  GFXcanvas16 *canvas = arcada.getCanvas();
  setHpLed(p -> getHp());
  
  // music main loop
  if (musicIsPlaying) {
    // fill music buffer if needed
    if (arcada.WavReadyForData()) {
      wavStatus status = arcada.WavReadFile();
    }
  } 
  //creating rooms
  switch (room){
    case 0: {

        //creating objects
        int objPos[3][2]= {{64,25},{10,50},{112,90}};
        canvas->drawRGBBitmap(0, 0, Map1, ARCADA_TFT_WIDTH, ARCADA_TFT_HEIGHT);
        p -> moveP(arcada, arcada.readButtons(), p , objPos, canvas);
        
        for(int i=0; i<(sizeof(objPos)/sizeof(objPos[0])); i++){  //draw objects
          canvas->drawRGBBitmap(objPos[i][0], objPos[i][1], stone, 16, 16);
        }  
        
        
        e1 -> moveE(p, e1, canvas);
        e2 -> moveE(p, e2, canvas);
        e3 -> moveE(p, e3, canvas);

        if(bState & ARCADA_BUTTONMASK_A){
          releaseA = true;
        }
        if(releaseA){
          p -> attackE( arcada, bState, p , e1, canvas);
          p -> attackE( arcada, bState, p , e2, canvas);
          p -> attackE( arcada, bState, p , e3, canvas);
          releaseA = false;
        }
        
        if( (e1 -> getEHp())==0 && (e2 -> getEHp())==0 && (e3 -> getEHp())==0){
          room=1;
        }

        if( (p -> getHp()) == 0){
          room=2;
        }
       break;   
    }
    //win screen
    case 1: {
      canvas->drawRGBBitmap(0, 0, Map1, ARCADA_TFT_WIDTH, ARCADA_TFT_HEIGHT);
      int objPos[][2]= {{}};
      p -> moveP(arcada, arcada.readButtons(), p , objPos, canvas);
      
      canvas->drawRGBBitmap(64, 48, coin, coin_mask, 32, 32);
      canvas->setTextColor(ARCADA_RED);
      canvas->setCursor(18, 30);
      canvas->print("You won!");

      if(arcada.readButtons() & ARCADA_BUTTONMASK_SELECT){
        (p -> setHP(11));
      }
      break;  
    }
    
    //death screen
    case 2: {
      canvas->fillScreen(ARCADA_BLACK);
      canvas->setTextColor(ARCADA_RED);
      canvas->setCursor(30, 60);
      canvas->print("YOU DIED");
      arcada.enableSpeaker(false);
      musicIsPlaying = false;
      musicFile.close();
      if(arcada.readButtons() & ARCADA_BUTTONMASK_SELECT){
        p -> setHP(1);
      }
      break;
    }
    
    default:
    {}
  }
}


//set healthpoints as LED
void setHpLed(int mcHP){
  pixels.begin();
  pixels.clear();
  pixels.setBrightness(1);
  switch (mcHP){
    case 0: 
      pixels.setPixelColor(0, pixels.Color(0, 0, 0));
      break;
    case 1: 
      pixels.setPixelColor(0, pixels.Color(255, 0, 0));
      break;
    case 2: 
      pixels.setPixelColor(0, pixels.Color(255, 255, 0));
      pixels.setPixelColor(1, pixels.Color(255, 255, 0));
      break;
    case 3: 
      pixels.setPixelColor(0, pixels.Color(255, 255, 0));
      pixels.setPixelColor(1, pixels.Color(255, 255, 0));
      pixels.setPixelColor(2, pixels.Color(255, 255, 0));
      break;
    case 4: 
      pixels.setPixelColor(0, pixels.Color(255, 255, 0));
      pixels.setPixelColor(1, pixels.Color(255, 255, 0));
      pixels.setPixelColor(2, pixels.Color(255, 255, 0));
      pixels.setPixelColor(3, pixels.Color(255, 255, 0));
      break;
    case 5: 
      pixels.setPixelColor(0, pixels.Color(255, 255, 0));
      pixels.setPixelColor(1, pixels.Color(255, 255, 0));
      pixels.setPixelColor(2, pixels.Color(255, 255, 0));
      pixels.setPixelColor(3, pixels.Color(255, 255, 0));
      pixels.setPixelColor(4, pixels.Color(255, 255, 0));
      break;
    case 6: 
      pixels.setPixelColor(0, pixels.Color(0, 255, 0));
      pixels.setPixelColor(1, pixels.Color(255, 255, 0));
      pixels.setPixelColor(2, pixels.Color(255, 255, 0));
      pixels.setPixelColor(3, pixels.Color(255, 255, 0));
      pixels.setPixelColor(4, pixels.Color(255, 255, 0));
      break;
    case 7: 
      pixels.setPixelColor(0, pixels.Color(0, 255, 0));
      pixels.setPixelColor(1, pixels.Color(0, 255, 0));
      pixels.setPixelColor(2, pixels.Color(255, 255, 0));
      pixels.setPixelColor(3, pixels.Color(255, 255, 0));
      pixels.setPixelColor(4, pixels.Color(255, 255, 0));
      break;
    case 8: 
      pixels.setPixelColor(0, pixels.Color(0, 255, 0));
      pixels.setPixelColor(1, pixels.Color(0, 255, 0));
      pixels.setPixelColor(2, pixels.Color(0, 255, 0));
      pixels.setPixelColor(3, pixels.Color(255, 255, 0));
      pixels.setPixelColor(4, pixels.Color(255, 255, 0));
      break;
    case 9: 
      pixels.setPixelColor(0, pixels.Color(0, 255, 0));
      pixels.setPixelColor(1, pixels.Color(0, 255, 0));
      pixels.setPixelColor(2, pixels.Color(0, 255, 0));
      pixels.setPixelColor(3, pixels.Color(0, 255, 0));
      pixels.setPixelColor(4, pixels.Color(255, 255, 0));
      break;
    case 10: 
      pixels.setPixelColor(0, pixels.Color(0, 255, 0));
      pixels.setPixelColor(1, pixels.Color(0, 255, 0));
      pixels.setPixelColor(2, pixels.Color(0, 255, 0));
      pixels.setPixelColor(3, pixels.Color(0, 255, 0));
      pixels.setPixelColor(4, pixels.Color(0, 255, 0));
      break;  
  }
  pixels.show();
}


/*
 * Prepares the PyBadge to play music
 */
void loadMusic() {
  // open the first wav file in the music folder
  musicFile = arcada.openFileByIndex("/music", 0, O_READ, "wav");
 
  Serial.println("Loading sound file...");
  
  uint32_t sampleRate;

  // load wav information, enable speakers and set interrupt for sampling
  wavStatus status = arcada.WavLoad(musicFile, &sampleRate);
  if ((status == WAV_LOAD) || (status == WAV_EOF)) {
    arcada.enableSpeaker(true);
    arcada.timerCallback(sampleRate, timedInterruptCallback);

    
    if (status == WAV_EOF && (p-> getHp()) > 0) {
      wavStatus status = arcada.WavLoad(musicFile, &sampleRate);
      if ((status == WAV_LOAD)){
        arcada.enableSpeaker(true);
         arcada.timerCallback(sampleRate, timedInterruptCallback);
         }

      }
  } 
}


/*
 * Interrupt for music sampling and game updates (called with 8kHz on our wav file - other sampling rates might not work)
 */
void timedInterruptCallback(void) {
  // we just entered the playing state
  //if (!musicIsPlaying) {
    //beatTime = millis() + bpmTiming;
  //}

  // play next sample
  wavStatus status = arcada.WavPlayNextSample();
  uint32_t sampleRate;

  // update playing state and hardware depending on wav playing status
  if((p -> getHp())== -1){
    arcada.timerStop();
    arcada.enableSpeaker(false);
    musicIsPlaying = false;
    inMenu = true;
    musicFile.close();
    setHpLed(p -> getHp());
    } 
  else if (status == WAV_EOF) {
    status = arcada.WavLoad(musicFile, &sampleRate);
  } else {
    musicIsPlaying = true;
  }
}


/*
 *create 2d array with enemy positions
 *y: number of enemies
 */
int** createEpos(int y){ 
  
  int** e_pos = new int*[y];
  
  for(int y = 0; y < 5; y++) {
    e_pos[y] = new int[2];
  }                                    

  return e_pos;
}
